#ifndef __JABY_ENGINE_PRIMITIVE_LINE_TYPES_HPP__
#define __JABY_ENGINE_PRIMITIVE_LINE_TYPES_HPP__
#include "primitive_support_types.hpp"

namespace JabyEngine {
    namespace GPU {
        namespace internal {
            struct LineCode : public CodeBase<LineCode> {
                static constexpr uint8_t CmdValue = 0b010;

                static constexpr auto GouraudShading  = Bit(28 - BitCorrection);
                static constexpr auto FlatShading     = !GouraudShading;
                static constexpr auto PolyLine        = Bit(27 - BitCorrection);
                static constexpr auto SingleLine      = !PolyLine;
                static constexpr auto SemiTransparent = Bit(25 - BitCorrection);
                static constexpr auto NonTransparent  = !SemiTransparent;
            };

            template<typename T>
            struct LineCodeInterface {
                typedef ::JabyEngine::GPU::internal::LineCode Code;

                constexpr T& set_poly_line(bool set = true) {
                    if(set) {
                        static_cast<T*>(this)->head.code.set(Code::PolyLine);
                    }
                    else {
                        static_cast<T*>(this)->head.code.set(Code::SingleLine);
                    }
                    return *static_cast<T*>(this);
                }

                constexpr T& set_semi_transparent(bool set = true) {
                    if(set) {
                        static_cast<T*>(this)->head.code.set(Code::SemiTransparent);
                    }
                    else {
                        static_cast<T*>(this)->head.code.set(Code::NonTransparent);
                    }
                    return *static_cast<T*>(this);
                }
            };
        }

        struct LINE_F : public internal::LineCodeInterface<LINE_F> {
            struct Head {
                static constexpr auto IdentityCode = Code().set(Code::FlatShading).set(Code::SingleLine).set(Code::NonTransparent);
            
                Color24 color;
                Code    code = IdentityCode;
            };

            typedef Vertex Body;

            union {
                Head   head;
                Vertex vertex;
            };

            static constexpr LINE_F new_line(const Color24& color, bool is_poly) {
                return LINE_F{.head = {.color = color, .code = Head::IdentityCode}}.set_poly_line(is_poly);
            }

            static constexpr LINE_F new_point(const Vertex& vertex) {
                return {.vertex = vertex};
            }

            static constexpr LINE_F new_terminate() {
                return {.vertex = {static_cast<short>(0xF000), static_cast<short>(0xF000u)}};
            }
        };

        struct LineMaker {
            // Make this it's own outside type??
            template<typename T, size_t N>
            struct Type {
                T head;
                T body[N];
            };

            static constexpr Type<LINE_F, 2> new_line_f(const Color24& color, const Vertex (&vertices)[2]) {
                return Type<LINE_F, 2>{.head = LINE_F::new_line(color, false), .body = {LINE_F::new_point(vertices[0]), LINE_F::new_point(vertices[1])}};
            }
        };
    }
}

#endif //!__JABY_ENGINE_PRIMITIVE_LINE_TYPES_HPP__